@@ -27,22 +27,34 @@ pub enum Language {
     Css,
     Dart,
     Elixir,
+    Elm,
     EmacsLisp,
+    Gleam,
     Go,
     Haskell,
+    Hcl,
+    Janet,
     Java,
     JavaScript,
     Json,
     Jsx,
+    Kotlin,
+    Lua,
+    Nix,
     OCaml,
     OCamlInterface,
     Php,
+    Perl,
     Python,
     Ruby,
     Rust,
     Scala,
+    Swift,
+    Toml,
     Tsx,
     TypeScript,
+    Yaml,
+    Zig,
 }
 
 use Language::*;
@@ -59,7 +71,10 @@ pub fn guess(path: &Path, src: &str) -> Option<Language> {
     }
 
     match path.extension() {
-        Some(extension) => from_extension(extension, src),
+        Some(extension) => match from_extension(extension) {
+            Some(Language::Php) if src.starts_with("<?hh") => None,
+            language => language,
+        },
         None => None,
     }
 }
@@ -91,20 +106,29 @@ fn from_emacs_mode_header(src: &str) -> Option<Language> {
             "dart" => Some(Dart),
             "c++" => Some(CPlusPlus),
             "elixir" => Some(Elixir),
+            "elm" => Some(Elm),
             "emacs-lisp" => Some(EmacsLisp),
+            "gleam" => Some(Gleam),
             "go" => Some(Go),
             "haskell" => Some(Haskell),
+            "hcl" => Some(Hcl),
+            "janet" => Some(Janet),
             "java" => Some(Java),
             "js" | "js2" => Some(JavaScript),
             "lisp" => Some(CommonLisp),
+            "perl" => Some(Perl),
             "python" => Some(Python),
             "rjsx" => Some(Jsx),
             "ruby" => Some(Ruby),
             "rust" => Some(Rust),
             "scala" => Some(Scala),
             "sh" => Some(Bash),
+            "swift" => Some(Swift),
+            "toml" => Some(Toml),
             "tuareg" => Some(OCaml),
             "typescript" => Some(TypeScript),
+            "yaml" => Some(Yaml),
+            "zig" => Some(Zig),
             _ => None,
         };
         if lang.is_some() {
@@ -135,8 +159,10 @@ fn from_shebang(src: &str) -> Option<Language> {
                     "chakra" | "d8" | "gjs" | "js" | "node" | "nodejs" | "qjs" | "rhino" | "v8"
                     | "v8-shell" => return Some(JavaScript),
                     "ocaml" | "ocamlrun" | "ocamlscript" => return Some(OCaml),
+                    "perl" => return Some(Perl),
                     "python" | "python2" | "python3" => return Some(Python),
                     "ruby" | "macruby" | "rake" | "jruby" | "rbx" => return Some(Ruby),
+                    "swift" => return Some(Swift),
                     "deno" | "ts-node" => return Some(TypeScript),
                     _ => {}
                 }
@@ -157,15 +183,19 @@ fn from_name(path: &Path) -> Option<Language> {
             | "gradlew" | "kshrc" | "login" | "man" | "profile" | "zlogin" | "zlogout"
             | "zprofile" | "zshenv" | "zshrc" => Some(Bash),
             ".emacs" | "_emacs" | "Cask" => Some(EmacsLisp),
+            ".arcconfig" | ".auto-changelog" | ".c8rc" | ".htmlhintrc" | ".imgbotconfig"
+            | ".nycrc" | ".tern-config" | ".tern-project" | ".watchmanconfig" | "Pipfile.lock"
+            | "composer.lock" | "mcmod.info" => Some(Json),
             "TARGETS" | "BUCK" | "DEPS" => Some(Python),
             "Gemfile" | "Rakefile" => Some(Ruby),
+            "Cargo.lock" | "Gopkg.lock" | "Pipfile" | "poetry.lock" => Some(Toml),
             _ => None,
         },
         None => None,
     }
 }
 
-fn from_extension(extension: &OsStr, src: &str) -> Option<Language> {
+pub fn from_extension(extension: &OsStr) -> Option<Language> {
     match extension.to_string_lossy().borrow() {
         "sh" | "bash" | "bats" | "cgi" | "command" | "env" | "fcgi" | "ksh" | "sh.in" | "tmux"
         | "tool" | "zsh" => Some(Bash),
@@ -173,9 +203,6 @@ fn from_extension(extension: &OsStr, src: &str) -> Option<Language> {
         // Treat .h as C++ rather than C. This is an arbitrary choice,
         // but C++ is more widely used than C according to
         // https://madnight.github.io/githut/
-        //
-        // TODO: allow users to override the association between
-        // extensions and parses.
         "cc" | "cpp" | "h" | "hh" | "hpp" | "cxx" => Some(CPlusPlus),
         "bb" | "boot" | "clj" | "cljc" | "clje" | "cljs" | "cljx" | "edn" | "joke" | "joker" => {
             Some(Clojure)
@@ -185,22 +212,37 @@ fn from_extension(extension: &OsStr, src: &str) -> Option<Language> {
         "css" => Some(Css),
         "dart" => Some(Dart),
         "el" => Some(EmacsLisp),
+        "elm" => Some(Elm),
         "ex" | "exs" => Some(Elixir),
+        "gleam" => Some(Gleam),
         "go" => Some(Go),
         "hs" => Some(Haskell),
+        "hcl" | "nomad" | "tf" | "tfvars" | "worfklow" => Some(Hcl),
+        "janet" | "jdn" => Some(Janet),
         "java" => Some(Java),
         "cjs" | "js" | "mjs" => Some(JavaScript),
         "jsx" => Some(Jsx),
-        "json" => Some(Json),
+        "json" | "avsc" | "geojson" | "gltf" | "har" | "ice" | "JSON-tmLanguage" | "jsonl"
+        | "mcmeta" | "tfstate" | "tfstate.backup" | "topojson" | "webapp" | "webmanifest" => {
+            Some(Json)
+        }
+        "kt" | "ktm" | "kts" => Some(Kotlin),
+        "lua" => Some(Lua),
+        "nix" => Some(Nix),
         "ml" => Some(OCaml),
         "mli" => Some(OCamlInterface),
-        "php" if !src.starts_with("<?hh") => Some(Php),
+        "php" => Some(Php),
+        "pm" | "pl" => Some(Perl),
         "py" | "py3" | "pyi" | "bzl" => Some(Python),
         "rb" | "builder" | "spec" | "rake" => Some(Ruby),
         "rs" => Some(Rust),
         "scala" | "sbt" | "sc" => Some(Scala),
+        "swift" => Some(Swift),
+        "toml" => Some(Toml),
         "ts" => Some(TypeScript),
         "tsx" => Some(Tsx),
+        "yaml" | "yml" => Some(Yaml),
+        "zig" => Some(Zig),
         _ => None,
     }
 }
@@ -247,7 +289,7 @@ mod tests {
     fn test_guess_by_emacs_mode_second_line() {
         let path = Path::new("foo");
         assert_eq!(
-            guess(path, "!#/bin/bash\n; -*- mode: Lisp; -*-"),
+            guess(path, "#!/bin/bash\n; -*- mode: Lisp; -*-"),
             Some(CommonLisp)
         );
     }
