@@ -13,6 +13,8 @@
 // the number of arguments and triggering this lint.
 #![allow(clippy::too_many_arguments)]
 
+mod changes;
+mod constants;
 mod context;
 mod dijkstra;
 mod files;
@@ -37,8 +39,9 @@ mod unchanged;
 extern crate log;
 
 use crate::hunks::{matched_pos_to_hunks, merge_adjacent};
+use changes::ChangeMap;
 use context::opposite_positions;
-use files::read_files_or_die;
+use files::{is_probably_binary, read_files_or_die, read_or_die, relative_paths_in_either};
 use guess_language::guess;
 use log::info;
 use mimalloc::MiMalloc;
@@ -50,21 +53,16 @@ use mimalloc::MiMalloc;
 #[global_allocator]
 static GLOBAL: MiMalloc = MiMalloc;
 
-use options::{should_use_color, Mode};
+use options::{DisplayMode, DisplayOptions, Mode};
+use rayon::prelude::*;
 use sliders::fix_all_sliders;
 use std::{env, path::Path};
-use style::BackgroundColor;
 use summary::{DiffResult, FileContent};
 use syntax::init_next_prev;
 use typed_arena::Arena;
-use walkdir::WalkDir;
 
 use crate::{
-    dijkstra::mark_syntax,
-    files::{is_probably_binary, read_or_die},
-    lines::MaxLine,
-    syntax::init_all_info,
-    tree_sitter_parser as tsp,
+    dijkstra::mark_syntax, lines::MaxLine, syntax::init_all_info, tree_sitter_parser as tsp,
 };
 
 extern crate pretty_env_logger;
@@ -84,15 +82,20 @@ fn reset_sigpipe() {
 
 /// The entrypoint.
 fn main() {
-    pretty_env_logger::init();
+    pretty_env_logger::init_timed();
     reset_sigpipe();
 
     match options::parse_args() {
-        Mode::DumpTreeSitter { path } => {
+        Mode::DumpTreeSitter {
+            path,
+            language_override,
+        } => {
             let path = Path::new(&path);
             let bytes = read_or_die(path);
             let src = String::from_utf8_lossy(&bytes).to_string();
-            match guess(path, &src) {
+
+            let language = language_override.or_else(|| guess(path, &src));
+            match language {
                 Some(lang) => {
                     let ts_lang = tsp::from_language(lang);
                     let tree = tsp::parse_to_tree(&src, &ts_lang);
@@ -103,12 +106,16 @@ fn main() {
                 }
             }
         }
-        Mode::DumpSyntax { path } => {
+        Mode::DumpSyntax {
+            path,
+            language_override,
+        } => {
             let path = Path::new(&path);
             let bytes = read_or_die(path);
             let src = String::from_utf8_lossy(&bytes).to_string();
 
-            match guess(path, &src) {
+            let language = language_override.or_else(|| guess(path, &src));
+            match language {
                 Some(lang) => {
                     let ts_lang = tsp::from_language(lang);
                     let arena = Arena::new();
@@ -124,49 +131,51 @@ fn main() {
         Mode::Diff {
             node_limit,
             byte_limit,
-            print_unchanged,
+            display_options,
             missing_as_empty,
-            background_color,
-            color_output,
-            display_width,
             display_path,
+            language_override,
             lhs_path,
             rhs_path,
-            ..
         } => {
-            let use_color = should_use_color(color_output);
-
             let lhs_path = Path::new(&lhs_path);
             let rhs_path = Path::new(&rhs_path);
 
+            if lhs_path == rhs_path {
+                eprintln!(
+                    "warning: You've specified the same {} twice.\n",
+                    if lhs_path.is_dir() {
+                        "directory"
+                    } else {
+                        "file"
+                    }
+                );
+            }
+
             if lhs_path.is_dir() && rhs_path.is_dir() {
-                for diff_result in
-                    diff_directories(lhs_path, rhs_path, missing_as_empty, node_limit, byte_limit)
-                {
-                    print_diff_result(
-                        display_width,
-                        use_color,
-                        background_color,
-                        print_unchanged,
-                        &diff_result,
-                    );
-                }
+                diff_directories(
+                    lhs_path,
+                    rhs_path,
+                    &display_options,
+                    node_limit,
+                    byte_limit,
+                    language_override,
+                )
+                .for_each(|diff_result| {
+                    print_diff_result(&display_options, &diff_result);
+                });
             } else {
                 let diff_result = diff_file(
                     &display_path,
                     lhs_path,
                     rhs_path,
+                    &display_options,
                     missing_as_empty,
                     node_limit,
                     byte_limit,
+                    language_override,
                 );
-                print_diff_result(
-                    display_width,
-                    use_color,
-                    background_color,
-                    print_unchanged,
-                    &diff_result,
-                );
+                print_diff_result(&display_options, &diff_result);
             }
         }
     };
@@ -177,20 +186,32 @@ fn diff_file(
     display_path: &str,
     lhs_path: &Path,
     rhs_path: &Path,
+    display_options: &DisplayOptions,
     missing_as_empty: bool,
     node_limit: u32,
     byte_limit: usize,
+    language_override: Option<guess_language::Language>,
 ) -> DiffResult {
     let (lhs_bytes, rhs_bytes) = read_files_or_die(lhs_path, rhs_path, missing_as_empty);
-    diff_file_content(display_path, &lhs_bytes, &rhs_bytes, node_limit, byte_limit)
+    diff_file_content(
+        display_path,
+        &lhs_bytes,
+        &rhs_bytes,
+        display_options.tab_width,
+        node_limit,
+        byte_limit,
+        language_override,
+    )
 }
 
 fn diff_file_content(
     display_path: &str,
     lhs_bytes: &[u8],
     rhs_bytes: &[u8],
+    tab_width: usize,
     node_limit: u32,
     byte_limit: usize,
+    language_override: Option<guess_language::Language>,
 ) -> DiffResult {
     if is_probably_binary(lhs_bytes) || is_probably_binary(rhs_bytes) {
         return DiffResult {
@@ -204,12 +225,23 @@ fn diff_file_content(
     }
 
     // TODO: don't replace tab characters inside string literals.
-    let lhs_src = String::from_utf8_lossy(lhs_bytes)
+    let tab_as_spaces = " ".repeat(tab_width);
+    let mut lhs_src = String::from_utf8_lossy(lhs_bytes)
         .to_string()
-        .replace('\t', "    ");
-    let rhs_src = String::from_utf8_lossy(rhs_bytes)
+        .replace('\t', &tab_as_spaces);
+    let mut rhs_src = String::from_utf8_lossy(rhs_bytes)
         .to_string()
-        .replace('\t', "    ");
+        .replace('\t', &tab_as_spaces);
+
+    // Ignore the trailing newline, if present.
+    // TODO: highlight if this has changes (#144).
+    // TODO: factor out a string cleaning function.
+    if lhs_src.ends_with('\n') {
+        lhs_src.pop();
+    }
+    if rhs_src.ends_with('\n') {
+        rhs_src.pop();
+    }
 
     // TODO: take a Path directly instead.
     let path = Path::new(&display_path);
@@ -222,14 +254,15 @@ fn diff_file_content(
     } else {
         &rhs_src
     };
-    let ts_lang = guess(path, guess_src).map(tsp::from_language);
+    let language = language_override.or_else(|| guess(path, guess_src));
+    let lang_config = language.map(tsp::from_language);
 
     if lhs_bytes == rhs_bytes {
         // If the two files are completely identical, return early
         // rather than doing any more work.
         return DiffResult {
             path: display_path.into(),
-            language: ts_lang.map(|l| l.name.into()),
+            language: lang_config.map(|l| l.name.into()),
             lhs_src: FileContent::Text("".into()),
             rhs_src: FileContent::Text("".into()),
             lhs_positions: vec![],
@@ -237,7 +270,7 @@ fn diff_file_content(
         };
     }
 
-    let (lang_name, lhs_positions, rhs_positions) = match ts_lang {
+    let (lang_name, lhs_positions, rhs_positions) = match lang_config {
         _ if lhs_bytes.len() > byte_limit || rhs_bytes.len() > byte_limit => {
             let lhs_positions = line_parser::change_positions(&lhs_src, &rhs_src);
             let rhs_positions = line_parser::change_positions(&rhs_src, &lhs_src);
@@ -254,10 +287,11 @@ fn diff_file_content(
 
             init_all_info(&lhs, &rhs);
 
+            let mut change_map = ChangeMap::default();
             let possibly_changed = if env::var("DFT_DBG_KEEP_UNCHANGED").is_ok() {
                 vec![(lhs.clone(), rhs.clone())]
             } else {
-                unchanged::mark_unchanged(&lhs, &rhs)
+                unchanged::mark_unchanged(&lhs, &rhs, &mut change_map)
             };
 
             let possibly_changed_max = max_num_nodes(&possibly_changed);
@@ -282,14 +316,16 @@ fn diff_file_content(
                     mark_syntax(
                         lhs_section_nodes.get(0).copied(),
                         rhs_section_nodes.get(0).copied(),
+                        &mut change_map,
                     );
 
-                    fix_all_sliders(&lhs_section_nodes);
-                    fix_all_sliders(&rhs_section_nodes);
+                    let language = language.unwrap();
+                    fix_all_sliders(language, &lhs_section_nodes, &mut change_map);
+                    fix_all_sliders(language, &rhs_section_nodes, &mut change_map);
                 }
 
-                let lhs_positions = syntax::change_positions(&lhs);
-                let rhs_positions = syntax::change_positions(&rhs);
+                let lhs_positions = syntax::change_positions(&lhs, &change_map);
+                let rhs_positions = syntax::change_positions(&rhs, &change_map);
                 (Some(ts_lang.name.into()), lhs_positions, rhs_positions)
             }
         }
@@ -311,49 +347,46 @@ fn diff_file_content(
 }
 
 /// Given two directories that contain the files, compare them
-/// pairwise.
+/// pairwise. Returns an iterator, so we can print results
+/// incrementally.
 ///
 /// When more than one file is modified, the hg extdiff extension passes directory
 /// paths with the all the modified files.
-fn diff_directories(
-    lhs_dir: &Path,
-    rhs_dir: &Path,
-    missing_as_empty: bool,
+fn diff_directories<'a>(
+    lhs_dir: &'a Path,
+    rhs_dir: &'a Path,
+    display_options: &DisplayOptions,
     node_limit: u32,
     byte_limit: usize,
-) -> Vec<DiffResult> {
-    let mut res = vec![];
-    for entry in WalkDir::new(lhs_dir).into_iter().filter_map(Result::ok) {
-        let lhs_path = entry.path();
-        if lhs_path.is_dir() {
-            continue;
-        }
+    language_override: Option<guess_language::Language>,
+) -> impl ParallelIterator<Item = DiffResult> + 'a {
+    let display_options = display_options.clone();
 
-        info!("LHS path is {:?} inside {:?}", lhs_path, lhs_dir);
+    // We greedily list all files in the directory, and then diff them
+    // in parallel. This is assuming that diffing is slower than
+    // enumerating files, so it benefits more from parallelism.
+    let paths = relative_paths_in_either(lhs_dir, rhs_dir);
 
-        let rel_path = lhs_path.strip_prefix(lhs_dir).unwrap();
-        let rhs_path = Path::new(rhs_dir).join(rel_path);
+    paths.into_par_iter().map(move |rel_path| {
+        info!("Relative path is {:?} inside {:?}", rel_path, lhs_dir);
 
-        res.push(diff_file(
+        let lhs_path = Path::new(lhs_dir).join(&rel_path);
+        let rhs_path = Path::new(rhs_dir).join(&rel_path);
+
+        diff_file(
             &rel_path.to_string_lossy(),
-            lhs_path,
+            &lhs_path,
             &rhs_path,
-            missing_as_empty,
+            &display_options,
+            true,
             node_limit,
             byte_limit,
-        ));
-    }
-    res
+            language_override,
+        )
+    })
 }
 
-// TODO: factor out a DiffOptions struct.
-fn print_diff_result(
-    display_width: usize,
-    use_color: bool,
-    background: BackgroundColor,
-    print_unchanged: bool,
-    summary: &DiffResult,
-) {
+fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
     match (&summary.lhs_src, &summary.rhs_src) {
         (FileContent::Text(lhs_src), FileContent::Text(rhs_src)) => {
             let opposite_to_lhs = opposite_positions(&summary.lhs_positions);
@@ -370,12 +403,19 @@ fn print_diff_result(
 
             let lang_name = summary.language.clone().unwrap_or_else(|| "Text".into());
             if hunks.is_empty() {
-                if print_unchanged {
+                if display_options.print_unchanged {
                     println!(
                         "{}",
-                        style::header(&summary.path, 1, 1, &lang_name, use_color, background)
+                        style::header(
+                            &summary.path,
+                            1,
+                            1,
+                            &lang_name,
+                            display_options.use_color,
+                            display_options.background_color
+                        )
                     );
-                    if lang_name == "Text" {
+                    if lang_name == "Text" || summary.lhs_src == summary.rhs_src {
                         // TODO: there are other Text names now, so
                         // they will hit the second case incorrectly.
                         println!("No changes.\n");
@@ -386,39 +426,46 @@ fn print_diff_result(
                 return;
             }
 
-            if env::var("INLINE").is_ok() {
-                inline::print(
-                    lhs_src,
-                    rhs_src,
-                    &summary.lhs_positions,
-                    &summary.rhs_positions,
-                    &hunks,
-                    &summary.path,
-                    &lang_name,
-                    use_color,
-                    background,
-                );
-            } else {
-                side_by_side::print(
-                    &hunks,
-                    display_width,
-                    use_color,
-                    background,
-                    &summary.path,
-                    &lang_name,
-                    lhs_src,
-                    rhs_src,
-                    &summary.lhs_positions,
-                    &summary.rhs_positions,
-                );
+            match display_options.display_mode {
+                DisplayMode::Inline => {
+                    inline::print(
+                        lhs_src,
+                        rhs_src,
+                        display_options,
+                        &summary.lhs_positions,
+                        &summary.rhs_positions,
+                        &hunks,
+                        &summary.path,
+                        &lang_name,
+                    );
+                }
+                DisplayMode::SideBySide | DisplayMode::SideBySideShowBoth => {
+                    side_by_side::print(
+                        &hunks,
+                        display_options,
+                        &summary.path,
+                        &lang_name,
+                        lhs_src,
+                        rhs_src,
+                        &summary.lhs_positions,
+                        &summary.rhs_positions,
+                    );
+                }
             }
         }
         (FileContent::Binary(lhs_bytes), FileContent::Binary(rhs_bytes)) => {
             let changed = lhs_bytes != rhs_bytes;
-            if print_unchanged || changed {
+            if display_options.print_unchanged || changed {
                 println!(
                     "{}",
-                    style::header(&summary.path, 1, 1, "binary", use_color, background)
+                    style::header(
+                        &summary.path,
+                        1,
+                        1,
+                        "binary",
+                        display_options.use_color,
+                        display_options.background_color
+                    )
                 );
                 if changed {
                     println!("Binary contents changed.");
@@ -431,7 +478,14 @@ fn print_diff_result(
             // We're diffing a binary file against a text file.
             println!(
                 "{}",
-                style::header(&summary.path, 1, 1, "binary", use_color, background)
+                style::header(
+                    &summary.path,
+                    1,
+                    1,
+                    "binary",
+                    display_options.use_color,
+                    display_options.background_color
+                )
             );
             println!("Binary contents changed.");
         }
@@ -464,7 +518,7 @@ fn max_num_nodes(roots_vec: &[(Vec<&syntax::Syntax>, Vec<&syntax::Syntax>)]) ->
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::options::{DEFAULT_BYTE_LIMIT, DEFAULT_NODE_LIMIT};
+    use crate::options::{DEFAULT_BYTE_LIMIT, DEFAULT_NODE_LIMIT, DEFAULT_TAB_WIDTH};
 
     #[test]
     fn test_diff_identical_content() {
@@ -473,8 +527,10 @@ mod tests {
             "foo.el",
             s.as_bytes(),
             s.as_bytes(),
+            DEFAULT_TAB_WIDTH,
             DEFAULT_NODE_LIMIT,
             DEFAULT_BYTE_LIMIT,
+            None,
         );
 
         assert_eq!(res.lhs_positions, vec![]);
