@@ -1,6 +1,7 @@
 //! Apply colours and styling to strings.
 
 use crate::{
+    constants::Side,
     lines::{byte_len, codepoint_len, LineNumber},
     positions::SingleLineSpan,
     syntax::{AtomKind, MatchKind, MatchedPos, TokenKind},
@@ -54,7 +55,7 @@ fn substring_by_byte(s: &str, start: usize, end: usize) -> &str {
 /// ```
 /// split_string_by_codepoint("fooba", 3) // vec!["foo", "ba "]
 /// ```
-fn split_string_by_codepoint(s: &str, max_len: usize) -> Vec<String> {
+fn split_string_by_codepoint(s: &str, max_len: usize, pad_last: bool) -> Vec<String> {
     let mut res = vec![];
     let mut s = s;
 
@@ -64,7 +65,11 @@ fn split_string_by_codepoint(s: &str, max_len: usize) -> Vec<String> {
     }
 
     if res.is_empty() || !s.is_empty() {
-        res.push(format!("{:width$}", s, width = max_len));
+        if pad_last {
+            res.push(format!("{:width$}", s, width = max_len));
+        } else {
+            res.push(s.to_string());
+        }
     }
 
     res
@@ -82,10 +87,11 @@ pub fn split_and_apply(
     max_len: usize,
     use_color: bool,
     styles: &[(SingleLineSpan, Style)],
+    side: Side,
 ) -> Vec<String> {
-    if styles.is_empty() && !line.is_empty() {
+    if styles.is_empty() && !line.trim().is_empty() {
         // Missing styles is a bug, so highlight in purple to make this obvious.
-        return split_string_by_codepoint(line, max_len)
+        return split_string_by_codepoint(line, max_len, matches!(side, Side::Left))
             .into_iter()
             .map(|part| {
                 if use_color {
@@ -100,7 +106,7 @@ pub fn split_and_apply(
     let mut styled_parts = vec![];
     let mut part_start = 0;
 
-    for part in split_string_by_codepoint(line, max_len) {
+    for part in split_string_by_codepoint(line, max_len, matches!(side, Side::Left)) {
         let mut res = String::with_capacity(part.len());
         let mut prev_style_end = 0;
         for (span, style) in styles {
@@ -236,6 +242,7 @@ pub fn novel_style(style: Style, is_lhs: bool, background: BackgroundColor) -> S
 pub fn color_positions(
     is_lhs: bool,
     background: BackgroundColor,
+    syntax_highlight: bool,
     positions: &[MatchedPos],
 ) -> Vec<(SingleLineSpan, Style)> {
     let mut styles = vec![];
@@ -243,39 +250,43 @@ pub fn color_positions(
         let mut style = Style::new();
         match pos.kind {
             MatchKind::UnchangedToken { highlight, .. } => {
-                if let TokenKind::Atom(atom_kind) = highlight {
-                    match atom_kind {
-                        AtomKind::String => {
-                            style = if background.is_dark() {
-                                style.bright_magenta()
-                            } else {
-                                style.magenta()
-                            };
+                if syntax_highlight {
+                    if let TokenKind::Atom(atom_kind) = highlight {
+                        match atom_kind {
+                            AtomKind::String => {
+                                style = if background.is_dark() {
+                                    style.bright_magenta()
+                                } else {
+                                    style.magenta()
+                                };
+                            }
+                            AtomKind::Comment => {
+                                style = style.italic();
+                                style = if background.is_dark() {
+                                    style.bright_blue()
+                                } else {
+                                    style.blue()
+                                };
+                            }
+                            AtomKind::Keyword | AtomKind::Type => {
+                                style = style.bold();
+                            }
+                            AtomKind::Normal => {}
                         }
-                        AtomKind::Comment => {
-                            style = style.italic();
-                            style = if background.is_dark() {
-                                style.bright_blue()
-                            } else {
-                                style.blue()
-                            };
-                        }
-                        AtomKind::Keyword | AtomKind::Type => {
-                            style = style.bold();
-                        }
-                        AtomKind::Normal => {}
                     }
                 }
             }
             MatchKind::Novel { highlight, .. } => {
                 style = novel_style(style, is_lhs, background);
-                if matches!(
-                    highlight,
-                    TokenKind::Delimiter
-                        | TokenKind::Atom(AtomKind::Keyword)
-                        | TokenKind::Atom(AtomKind::Type)
-                ) {
-                    style = style.bold();
+                if syntax_highlight {
+                    if matches!(
+                        highlight,
+                        TokenKind::Delimiter
+                            | TokenKind::Atom(AtomKind::Keyword)
+                            | TokenKind::Atom(AtomKind::Type)
+                    ) {
+                        style = style.bold();
+                    }
                 }
                 if matches!(highlight, TokenKind::Atom(AtomKind::Comment)) {
                     style = style.italic();
@@ -283,14 +294,18 @@ pub fn color_positions(
             }
             MatchKind::NovelWord { highlight } => {
                 style = novel_style(style, is_lhs, background).bold();
-                if matches!(highlight, TokenKind::Atom(AtomKind::Comment)) {
-                    style = style.italic();
+                if syntax_highlight {
+                    if matches!(highlight, TokenKind::Atom(AtomKind::Comment)) {
+                        style = style.italic();
+                    }
                 }
             }
             MatchKind::NovelLinePart { highlight, .. } => {
                 style = novel_style(style, is_lhs, background);
-                if matches!(highlight, TokenKind::Atom(AtomKind::Comment)) {
-                    style = style.italic();
+                if syntax_highlight {
+                    if matches!(highlight, TokenKind::Atom(AtomKind::Comment)) {
+                        style = style.italic();
+                    }
                 }
             }
         };
@@ -302,10 +317,11 @@ pub fn color_positions(
 pub fn apply_colors(
     s: &str,
     is_lhs: bool,
+    syntax_highlight: bool,
     background: BackgroundColor,
     positions: &[MatchedPos],
 ) -> String {
-    let styles = color_positions(is_lhs, background, positions);
+    let styles = color_positions(is_lhs, background, syntax_highlight, positions);
     apply(s, &styles)
 }
 
@@ -328,10 +344,13 @@ pub fn header(
     } else {
         file_name.to_string()
     };
-    format!(
-        "{} --- {}/{} --- {}",
-        file_name_pretty, hunk_num, hunk_total, language_name
-    )
+    let divider = if hunk_total == 1 {
+        "".to_owned()
+    } else {
+        format!("{}/{} --- ", hunk_num, hunk_total)
+    };
+
+    format!("{} --- {}{}", file_name_pretty, divider, language_name)
 }
 
 #[cfg(test)]
@@ -351,17 +370,31 @@ mod tests {
 
     #[test]
     fn split_string_simple() {
-        assert_eq!(split_string_by_codepoint("fooba", 3), vec!["foo", "ba "]);
+        assert_eq!(
+            split_string_by_codepoint("fooba", 3, true),
+            vec!["foo", "ba "]
+        );
+    }
+
+    #[test]
+    fn split_string_simple_no_pad() {
+        assert_eq!(
+            split_string_by_codepoint("fooba", 3, false),
+            vec!["foo", "ba"]
+        );
     }
 
     #[test]
     fn split_string_unicode() {
-        assert_eq!(split_string_by_codepoint("ab📦def", 3), vec!["ab📦", "def"]);
+        assert_eq!(
+            split_string_by_codepoint("ab📦def", 3, true),
+            vec!["ab📦", "def"]
+        );
     }
 
     #[test]
     fn test_split_and_apply_missing() {
-        let res = split_and_apply("foo", 3, true, &[]);
+        let res = split_and_apply("foo", 3, true, &[], Side::Left);
         assert_eq!(res, vec![highlight_missing_style_bug("foo")])
     }
 
@@ -379,6 +412,7 @@ mod tests {
                 },
                 Style::new(),
             )],
+            Side::Left,
         );
         assert_eq!(res, vec!["foo"])
     }
@@ -397,6 +431,7 @@ mod tests {
                 },
                 Style::new(),
             )],
+            Side::Left,
         );
         assert_eq!(res, vec!["foobar"])
     }
@@ -425,6 +460,7 @@ mod tests {
                     Style::new(),
                 ),
             ],
+            Side::Left,
         );
         assert_eq!(res, vec!["foo", "bar"])
     }
@@ -443,6 +479,7 @@ mod tests {
                 },
                 Style::new(),
             )],
+            Side::Left,
         );
         assert_eq!(res, vec!["foobar", "      "])
     }
