@@ -2,30 +2,18 @@
 
 #![allow(clippy::mutable_key_type)] // Hash for Syntax doesn't use mutable fields.
 
-use std::{
-    cell::Cell,
-    collections::HashMap,
-    env, fmt,
-    hash::{Hash, Hasher},
-    num::NonZeroU32,
-};
+use std::{cell::Cell, collections::HashMap, env, fmt, hash::Hash, num::NonZeroU32};
 use typed_arena::Arena;
 
 use crate::{
+    changes::ChangeKind,
+    changes::{ChangeKind::*, ChangeMap},
     lines::{LineNumber, NewlinePositions},
     myers_diff,
     positions::SingleLineSpan,
 };
-use ChangeKind::*;
 use Syntax::*;
 
-#[derive(PartialEq, Eq, Clone, Copy)]
-pub enum ChangeKind<'a> {
-    Unchanged(&'a Syntax<'a>),
-    ReplacedComment(&'a Syntax<'a>, &'a Syntax<'a>),
-    Novel,
-}
-
 /// A Debug implementation that does not reucurse into the
 /// corresponding node mentioned for Unchanged. Otherwise we will
 /// infinitely loop on unchanged nodes, which both point to the other.
@@ -60,9 +48,6 @@ pub struct SyntaxInfo<'a> {
     /// Does the previous syntax node occur on the same line as the
     /// first line of this node?
     prev_is_contiguous: Cell<bool>,
-    /// Whether or not this syntax node has changed. This value is set
-    /// when computing the diff with another syntax tree.
-    pub change: Cell<Option<ChangeKind<'a>>>,
     /// The number of nodes that are ancestors of this one.
     num_ancestors: Cell<u32>,
     /// A number that uniquely identifies this syntax node.
@@ -83,7 +68,6 @@ impl<'a> SyntaxInfo<'a> {
             prev: Cell::new(None),
             parent: Cell::new(None),
             prev_is_contiguous: Cell::new(false),
-            change: Cell::new(None),
             num_ancestors: Cell::new(0),
             unique_id: Cell::new(NonZeroU32::new(u32::MAX).unwrap()),
             content_id: Cell::new(0),
@@ -151,8 +135,6 @@ impl<'a> fmt::Debug for Syntax<'a> {
                     .field("close_position", &dbg_pos(close_position));
 
                 if env::var("DFT_VERBOSE").is_ok() {
-                    ds.field("change", &info.change.get());
-
                     let next_sibling_s = match info.next_sibling.get() {
                         Some(List { .. }) => "Some(List)",
                         Some(Atom { .. }) => "Some(Atom)",
@@ -180,7 +162,6 @@ impl<'a> fmt::Debug for Syntax<'a> {
 
                 if env::var("DFT_VERBOSE").is_ok() {
                     ds.field("highlight", highlight);
-                    ds.field("change", &info.change.get());
                     let next_sibling_s = match info.next_sibling.get() {
                         Some(List { .. }) => "Some(List)",
                         Some(Atom { .. }) => "Some(Atom)",
@@ -333,36 +314,6 @@ impl<'a> Syntax<'a> {
         };
         position.last().map(|lp| lp.line)
     }
-
-    pub fn change(&'a self) -> Option<ChangeKind<'a>> {
-        self.info().change.get()
-    }
-
-    pub fn set_change(&self, ck: ChangeKind<'a>) {
-        self.info().change.set(Some(ck));
-    }
-
-    pub fn set_change_deep(&self, ck: ChangeKind<'a>) {
-        self.set_change(ck);
-
-        if let List { children, .. } = self {
-            // For unchanged lists, match up children with the
-            // unchanged children on the other side.
-            if let Unchanged(List {
-                children: other_children,
-                ..
-            }) = ck
-            {
-                for (child, other_child) in children.iter().zip(other_children) {
-                    child.set_change_deep(Unchanged(other_child));
-                }
-            } else {
-                for child in children {
-                    child.set_change_deep(ck);
-                }
-            };
-        }
-    }
 }
 
 /// Initialise all the fields in `SyntaxInfo`.
@@ -555,12 +506,6 @@ impl<'a> PartialEq for Syntax<'a> {
 }
 impl<'a> Eq for Syntax<'a> {}
 
-impl<'a> Hash for Syntax<'a> {
-    fn hash<H: Hasher>(&self, state: &mut H) {
-        self.content_id().hash(state);
-    }
-}
-
 #[derive(PartialEq, Eq, Debug, Clone, Copy, Hash)]
 pub enum AtomKind {
     Normal,
@@ -807,16 +752,23 @@ impl MatchedPos {
 }
 
 /// Walk `nodes` and return a vec of all the changed positions.
-pub fn change_positions<'a>(nodes: &[&'a Syntax<'a>]) -> Vec<MatchedPos> {
+pub fn change_positions<'a>(
+    nodes: &[&'a Syntax<'a>],
+    change_map: &ChangeMap<'a>,
+) -> Vec<MatchedPos> {
     let mut positions = Vec::new();
-    change_positions_(nodes, &mut positions);
+    change_positions_(nodes, change_map, &mut positions);
     positions
 }
 
-fn change_positions_<'a>(nodes: &[&'a Syntax<'a>], positions: &mut Vec<MatchedPos>) {
+fn change_positions_<'a>(
+    nodes: &[&'a Syntax<'a>],
+    change_map: &ChangeMap<'a>,
+    positions: &mut Vec<MatchedPos>,
+) {
     for node in nodes {
-        let change = node
-            .change()
+        let change = change_map
+            .get(node)
             .unwrap_or_else(|| panic!("Should have changes set in all nodes: {:#?}", node));
 
         match node {
@@ -833,7 +785,7 @@ fn change_positions_<'a>(nodes: &[&'a Syntax<'a>], positions: &mut Vec<MatchedPo
                     false,
                 ));
 
-                change_positions_(children, positions);
+                change_positions_(children, change_map, positions);
 
                 positions.extend(MatchedPos::new(
                     change,
@@ -998,40 +950,6 @@ mod tests {
         assert_eq!(x, y);
     }
 
-    #[test]
-    fn test_atom_equality_ignores_change() {
-        let lhs = Atom {
-            info: SyntaxInfo {
-                change: Cell::new(Some(Novel)),
-                ..SyntaxInfo::default()
-            },
-
-            position: vec![SingleLineSpan {
-                line: 1.into(),
-                start_col: 2,
-                end_col: 3,
-            }],
-            content: "foo".into(),
-            kind: AtomKind::Normal,
-        };
-        let rhs = Atom {
-            info: SyntaxInfo {
-                change: Cell::new(None),
-                ..SyntaxInfo::default()
-            },
-            position: vec![SingleLineSpan {
-                line: 1.into(),
-                start_col: 2,
-                end_col: 3,
-            }],
-            content: "foo".into(),
-            kind: AtomKind::Normal,
-        };
-        init_all_info(&[&lhs], &[&rhs]);
-
-        assert_eq!(lhs, rhs);
-    }
-
     #[test]
     fn test_split_comment_words_basic() {
         let content = "abc";
