@@ -50,22 +50,35 @@ extern "C" {
     fn tree_sitter_cpp() -> ts::Language;
     fn tree_sitter_commonlisp() -> ts::Language;
     fn tree_sitter_css() -> ts::Language;
+    fn tree_sitter_dart() -> ts::Language;
     fn tree_sitter_elisp() -> ts::Language;
     fn tree_sitter_elixir() -> ts::Language;
+    fn tree_sitter_gleam() -> ts::Language;
+    fn tree_sitter_elm() -> ts::Language;
     fn tree_sitter_go() -> ts::Language;
     fn tree_sitter_haskell() -> ts::Language;
+    fn tree_sitter_hcl() -> ts::Language;
+    fn tree_sitter_janet_simple() -> ts::Language;
     fn tree_sitter_java() -> ts::Language;
     fn tree_sitter_javascript() -> ts::Language;
     fn tree_sitter_json() -> ts::Language;
+    fn tree_sitter_kotlin() -> ts::Language;
+    fn tree_sitter_lua() -> ts::Language;
+    fn tree_sitter_nix() -> ts::Language;
     fn tree_sitter_ocaml() -> ts::Language;
     fn tree_sitter_ocaml_interface() -> ts::Language;
     fn tree_sitter_php() -> ts::Language;
+    fn tree_sitter_perl() -> ts::Language;
     fn tree_sitter_python() -> ts::Language;
     fn tree_sitter_ruby() -> ts::Language;
     fn tree_sitter_rust() -> ts::Language;
     fn tree_sitter_scala() -> ts::Language;
+    fn tree_sitter_swift() -> ts::Language;
+    fn tree_sitter_toml() -> ts::Language;
     fn tree_sitter_tsx() -> ts::Language;
     fn tree_sitter_typescript() -> ts::Language;
+    fn tree_sitter_yaml() -> ts::Language;
+    fn tree_sitter_zig() -> ts::Language;
 }
 
 // TODO: begin/end and object/end.
@@ -86,10 +99,10 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Bash",
                 language,
-                atom_nodes: (vec!["string", "raw_string", "heredoc_body"])
+                atom_nodes: vec!["string", "raw_string", "heredoc_body"]
                     .into_iter()
                     .collect(),
-                delimiter_tokens: (vec![("(", ")"), ("{", "}"), ("[", "]")]),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/bash.scm"),
@@ -102,10 +115,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "C",
                 language,
-                atom_nodes: (vec!["string_literal", "char_literal"])
-                    .into_iter()
-                    .collect(),
-                delimiter_tokens: (vec![("(", ")"), ("{", "}"), ("[", "]")]),
+                atom_nodes: vec!["string_literal", "char_literal"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/c.scm"),
@@ -120,10 +131,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 language,
                 // The C++ grammar extends the C grammar, so the node
                 // names are generally the same.
-                atom_nodes: (vec!["string_literal", "char_literal"])
-                    .into_iter()
-                    .collect(),
-                delimiter_tokens: (vec![("(", ")"), ("{", "}"), ("[", "]"), ("<", ">")]),
+                atom_nodes: vec!["string_literal", "char_literal"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]"), ("<", ">")],
                 highlight_query: ts::Query::new(
                     language,
                     concat!(
@@ -139,11 +148,15 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Clojure",
                 language,
-                atom_nodes: (vec![]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]")])
+                atom_nodes: vec![].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]")]
                     .into_iter()
                     .collect(),
-                highlight_query: ts::Query::new(language, "").unwrap(),
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/clojure.scm"),
+                )
+                .unwrap(),
             }
         }
         CommonLisp => {
@@ -151,8 +164,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Common Lisp",
                 language,
-                atom_nodes: (vec!["str_lit", "char_lit"]).into_iter().collect(),
-                delimiter_tokens: (vec![("(", ")")]),
+                atom_nodes: vec!["str_lit", "char_lit"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")")],
                 highlight_query: ts::Query::new(language, "").unwrap(),
             }
         }
@@ -161,14 +174,14 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "C#",
                 language,
-                atom_nodes: (vec![
+                atom_nodes: vec![
                     "string_literal",
                     "verbatim_string_literal",
                     "character_literal",
-                ])
+                ]
                 .into_iter()
                 .collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")")]),
+                delimiter_tokens: vec![("{", "}"), ("(", ")")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/c-sharp.scm"),
@@ -181,8 +194,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "CSS",
                 language,
-                atom_nodes: (vec!["integer_value", "float_value"]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")")]),
+                atom_nodes: vec!["integer_value", "float_value"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/css.scm"),
@@ -190,13 +203,27 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 .unwrap(),
             }
         }
+        Dart => {
+            let language = unsafe { tree_sitter_dart() };
+            TreeSitterConfig {
+                name: "Dart",
+                language,
+                atom_nodes: vec!["string_literal", "script_tag"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")],
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/dart.scm"),
+                )
+                .unwrap(),
+            }
+        }
         EmacsLisp => {
             let language = unsafe { tree_sitter_elisp() };
             TreeSitterConfig {
                 name: "Emacs Lisp",
                 language,
-                atom_nodes: (vec![]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]")])
+                atom_nodes: vec![].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]")]
                     .into_iter()
                     .collect(),
                 highlight_query: ts::Query::new(
@@ -211,8 +238,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Elixir",
                 language,
-                atom_nodes: (vec!["string", "heredoc"]).into_iter().collect(),
-                delimiter_tokens: (vec![("(", ")"), ("{", "}"), ("do", "end")])
+                atom_nodes: vec!["string", "heredoc"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("do", "end")]
                     .into_iter()
                     .collect(),
                 highlight_query: ts::Query::new(
@@ -222,15 +249,43 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 .unwrap(),
             }
         }
+        Gleam => {
+            let language = unsafe { tree_sitter_gleam() };
+            TreeSitterConfig {
+                name: "Gleam",
+                language,
+                atom_nodes: ["string"].into(),
+                delimiter_tokens: vec![("(", ")"), ("[", "]"), ("{", "}")],
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/gleam.scm"),
+                )
+                .unwrap(),
+            }
+        }
+        Elm => {
+            let language = unsafe { tree_sitter_elm() };
+            TreeSitterConfig {
+                name: "Elm",
+                language,
+                atom_nodes: vec!["string_constant_expr"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("[", "]"), ("(", ")")],
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/elm.scm"),
+                )
+                .unwrap(),
+            }
+        }
         Go => {
             let language = unsafe { tree_sitter_go() };
             TreeSitterConfig {
                 name: "Go",
                 language,
-                atom_nodes: (vec!["interpreted_string_literal", "raw_string_literal"])
+                atom_nodes: vec!["interpreted_string_literal", "raw_string_literal"]
                     .into_iter()
                     .collect(),
-                delimiter_tokens: (vec![("{", "}"), ("[", "]"), ("(", ")")])
+                delimiter_tokens: vec![("{", "}"), ("[", "]"), ("(", ")")]
                     .into_iter()
                     .collect(),
                 highlight_query: ts::Query::new(
@@ -245,8 +300,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Haskell",
                 language,
-                atom_nodes: (vec![]).into_iter().collect(),
-                delimiter_tokens: (vec![("[", "]"), ("(", ")")]),
+                atom_nodes: vec![].into_iter().collect(),
+                delimiter_tokens: vec![("[", "]"), ("(", ")")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/haskell.scm"),
@@ -254,13 +309,57 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 .unwrap(),
             }
         }
+        Hcl => {
+            let language = unsafe { tree_sitter_hcl() };
+            TreeSitterConfig {
+                name: "Hcl",
+                language,
+                atom_nodes: vec!["string_lit", "heredoc_template"].into_iter().collect(),
+                delimiter_tokens: vec![
+                    ("[", "]"),
+                    ("(", ")"),
+                    ("{", "}"),
+                    ("%{", "}"),
+                    ("%{~", "~}"),
+                    ("${", "}"),
+                ],
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/hcl.scm"),
+                )
+                .unwrap(),
+            }
+        }
+        Janet => {
+            let language = unsafe { tree_sitter_janet_simple() };
+            TreeSitterConfig {
+                name: "Janet",
+                language,
+                atom_nodes: vec![].into_iter().collect(),
+                delimiter_tokens: vec![
+                    ("@{", "}"),
+                    ("@(", ")"),
+                    ("@[", "]"),
+                    ("{", "}"),
+                    ("(", ")"),
+                    ("[", "]"),
+                ]
+                .into_iter()
+                .collect(),
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/janet_simple.scm"),
+                )
+                .unwrap(),
+            }
+        }
         Java => {
             let language = unsafe { tree_sitter_java() };
             TreeSitterConfig {
                 name: "Java",
                 language,
-                atom_nodes: (vec![]).into_iter().collect(),
-                delimiter_tokens: (vec![("(", ")"), ("{", "}")]),
+                atom_nodes: vec![].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/java.scm"),
@@ -273,10 +372,10 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "JavaScript",
                 language,
-                atom_nodes: (vec!["string", "template_string", "regex"])
+                atom_nodes: vec!["string", "template_string", "regex"]
                     .into_iter()
                     .collect(),
-                delimiter_tokens: (vec![
+                delimiter_tokens: vec![
                     ("[", "]"),
                     ("(", ")"),
                     ("{", "}"),
@@ -284,7 +383,7 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                     // like 1 < 2, but we should never see both a < and a
                     // > at the same level in JSX.
                     ("<", ">"),
-                ]),
+                ],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/javascript.scm"),
@@ -297,8 +396,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "JSON",
                 language,
-                atom_nodes: (vec!["string"]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("[", "]")]),
+                atom_nodes: vec!["string"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("[", "]")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/json.scm"),
@@ -306,13 +405,63 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 .unwrap(),
             }
         }
+        Kotlin => {
+            let language = unsafe { tree_sitter_kotlin() };
+            TreeSitterConfig {
+                name: "Kotlin",
+                language,
+                atom_nodes: vec!["line_string_literal", "character_literal"]
+                    .into_iter()
+                    .collect(),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]"), ("<", ">")]
+                    .into_iter()
+                    .collect(),
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/kotlin.scm"),
+                )
+                .unwrap(),
+            }
+        }
+        Lua => {
+            let language = unsafe { tree_sitter_lua() };
+            TreeSitterConfig {
+                name: "Lua",
+                language,
+                atom_nodes: vec!["string"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]")]
+                    .into_iter()
+                    .collect(),
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/lua.scm"),
+                )
+                .unwrap(),
+            }
+        }
+        Nix => {
+            let language = unsafe { tree_sitter_nix() };
+            TreeSitterConfig {
+                name: "Nix",
+                language,
+                atom_nodes: vec!["string_expression", "indented_string_expression"]
+                    .into_iter()
+                    .collect(),
+                delimiter_tokens: vec![("{", "}"), ("[", "]")].into_iter().collect(),
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/nix.scm"),
+                )
+                .unwrap(),
+            }
+        }
         OCaml => {
             let language = unsafe { tree_sitter_ocaml() };
             TreeSitterConfig {
                 name: "OCaml",
                 language,
                 atom_nodes: OCAML_ATOM_NODES.iter().copied().collect(),
-                delimiter_tokens: (vec![("(", ")"), ("[", "]"), ("{", "}")]),
+                delimiter_tokens: vec![("(", ")"), ("[", "]"), ("{", "}")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/ocaml.scm"),
@@ -326,7 +475,7 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 name: "OCaml Interface",
                 language,
                 atom_nodes: OCAML_ATOM_NODES.iter().copied().collect(),
-                delimiter_tokens: (vec![("(", ")"), ("[", "]"), ("{", "}")]),
+                delimiter_tokens: vec![("(", ")"), ("[", "]"), ("{", "}")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/ocaml.scm"),
@@ -339,8 +488,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "PHP",
                 language,
-                atom_nodes: (vec!["string"]).into_iter().collect(),
-                delimiter_tokens: (vec![("(", ")"), ("[", "]"), ("{", "}")]),
+                atom_nodes: vec!["string", "encapsed_string"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("[", "]"), ("{", "}")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/php.scm"),
@@ -348,13 +497,29 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 .unwrap(),
             }
         }
+        Perl => {
+            let language = unsafe { tree_sitter_perl() };
+            TreeSitterConfig {
+                name: "Perl",
+                language,
+                atom_nodes: vec!["string_single_quoted", "string_double_quoted"]
+                    .into_iter()
+                    .collect(),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]")],
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/perl.scm"),
+                )
+                .unwrap(),
+            }
+        }
         Python => {
             let language = unsafe { tree_sitter_python() };
             TreeSitterConfig {
                 name: "Python",
                 language,
-                atom_nodes: (vec!["string"]).into_iter().collect(),
-                delimiter_tokens: (vec![("(", ")"), ("[", "]"), ("{", "}")]),
+                atom_nodes: vec!["string"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("[", "]"), ("{", "}")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/python.scm"),
@@ -367,10 +532,10 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Ruby",
                 language,
-                atom_nodes: (vec!["string", "heredoc_body", "regex"])
+                atom_nodes: vec!["string", "heredoc_body", "regex"]
                     .into_iter()
                     .collect(),
-                delimiter_tokens: (vec![
+                delimiter_tokens: vec![
                     ("{", "}"),
                     ("(", ")"),
                     ("[", "]"),
@@ -378,7 +543,7 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                     ("def", "end"),
                     ("begin", "end"),
                     ("class", "end"),
-                ]),
+                ],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/ruby.scm"),
@@ -391,16 +556,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Rust",
                 language,
-                atom_nodes: (vec!["char_literal", "string_literal"])
-                    .into_iter()
-                    .collect(),
-                delimiter_tokens: (vec![
-                    ("{", "}"),
-                    ("(", ")"),
-                    ("[", "]"),
-                    ("|", "|"),
-                    ("<", ">"),
-                ]),
+                atom_nodes: vec!["char_literal", "string_literal"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]"), ("|", "|"), ("<", ">")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/rust.scm"),
@@ -413,20 +570,50 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Scala",
                 language,
-                atom_nodes: (vec!["string", "template_string"]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]")]),
-                highlight_query: ts::Query::new(language, "").unwrap(),
+                atom_nodes: vec!["string", "template_string"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]")],
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/scala.scm"),
+                )
+                .unwrap(),
             }
         }
-        TypeScript => {
-            let language = unsafe { tree_sitter_typescript() };
+        Swift => {
+            let language = unsafe { tree_sitter_swift() };
             TreeSitterConfig {
-                name: "TypeScript",
+                name: "Swift",
                 language,
-                atom_nodes: (vec!["string", "template_string", "regex"])
-                    .into_iter()
-                    .collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")]),
+                atom_nodes: ["line_string_literal"].into(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")],
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/swift.scm"),
+                )
+                .unwrap(),
+            }
+        }
+        Toml => {
+            let language = unsafe { tree_sitter_toml() };
+            TreeSitterConfig {
+                name: "TOML",
+                language,
+                atom_nodes: vec!["string"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("[", "]")],
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/toml.scm"),
+                )
+                .unwrap(),
+            }
+        }
+        Tsx => {
+            let language = unsafe { tree_sitter_tsx() };
+            TreeSitterConfig {
+                name: "TypeScript TSX",
+                language,
+                atom_nodes: vec!["string", "template_string"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")],
                 highlight_query: ts::Query::new(
                     language,
                     concat!(
@@ -437,13 +624,15 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 .unwrap(),
             }
         }
-        Tsx => {
-            let language = unsafe { tree_sitter_tsx() };
+        TypeScript => {
+            let language = unsafe { tree_sitter_typescript() };
             TreeSitterConfig {
-                name: "TypeScript TSX",
+                name: "TypeScript",
                 language,
-                atom_nodes: (vec!["string", "template_string"]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")]),
+                atom_nodes: vec!["string", "template_string", "regex"]
+                    .into_iter()
+                    .collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")],
                 highlight_query: ts::Query::new(
                     language,
                     concat!(
@@ -454,6 +643,45 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 .unwrap(),
             }
         }
+        Yaml => {
+            let language = unsafe { tree_sitter_yaml() };
+            TreeSitterConfig {
+                name: "YAML",
+                language,
+                atom_nodes: vec![
+                    "string_scalar",
+                    "double_quote_scalar",
+                    "single_quote_scalar",
+                    "block_scalar",
+                ]
+                .into_iter()
+                .collect(),
+                delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]")]),
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/yaml.scm"),
+                )
+                .unwrap(),
+            }
+        }
+        Zig => {
+            let language = unsafe { tree_sitter_zig() };
+            TreeSitterConfig {
+                name: "Zig",
+                language,
+                atom_nodes: vec!["STRINGLITERALSINGLE", "BUILTINIDENTIFIER"]
+                    .into_iter()
+                    .collect(),
+                delimiter_tokens: (vec![("{", "}"), ("[", "]"), ("(", ")")])
+                    .into_iter()
+                    .collect(),
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/zig.scm"),
+                )
+                .unwrap(),
+            }
+        }
     }
 }
 
@@ -474,45 +702,51 @@ fn tree_highlights(
     src: &str,
     config: &TreeSitterConfig,
 ) -> HighlightedNodeIds {
-    let mut keyword_ish_capture_ids = vec![];
-    if let Some(idx) = config.highlight_query.capture_index_for_name("keyword") {
-        keyword_ish_capture_ids.push(idx);
-    }
-    if let Some(idx) = config.highlight_query.capture_index_for_name("operator") {
-        keyword_ish_capture_ids.push(idx);
-    }
-    if let Some(idx) = config.highlight_query.capture_index_for_name("constant") {
-        keyword_ish_capture_ids.push(idx);
-    }
-    if let Some(idx) = config
-        .highlight_query
-        .capture_index_for_name("constant.builtin")
-    {
-        keyword_ish_capture_ids.push(idx);
-    }
-
+    let mut keyword_ish_capture_ids: Vec<u32> = vec![];
     let mut string_capture_ids = vec![];
-    if let Some(idx) = config.highlight_query.capture_index_for_name("string") {
-        string_capture_ids.push(idx);
-    }
-
     let mut type_capture_ids = vec![];
-    if let Some(idx) = config.highlight_query.capture_index_for_name("type") {
-        type_capture_ids.push(idx);
-    }
-    if let Some(idx) = config
-        .highlight_query
-        .capture_index_for_name("type.builtin")
-    {
-        type_capture_ids.push(idx);
-    }
-    if let Some(idx) = config.highlight_query.capture_index_for_name("label") {
+
+    // Query names are often written with namespacing, so
+    // highlights.scm might contain @constant or the more specific
+    // @constant.builtin.
+    //
+    // We support e.g. arbitrary @constant.foo so we get the benefit
+    // of all the relevant highlighting queries.
+    let cn = config.highlight_query.capture_names();
+    for (idx, name) in cn.iter().enumerate() {
+        if name == "type"
+            || name.starts_with("type.")
+            || name.starts_with("storage.type.")
+            || name.starts_with("keyword.type.")
+            || name == "tag"
+        {
+            // TODO: this doesn't capture (type_ref) in Elm as that
+            // applies to the parent node.
+            type_capture_ids.push(idx as u32);
+        } else if name == "keyword"
+            || name.starts_with("keyword.")
+            || name == "constant"
+            || name.starts_with("constant.")
+            || name == "operator"
+            || name == "repeat"
+            || name == "boolean"
+        {
+            keyword_ish_capture_ids.push(idx as u32);
+        }
+
+        if name == "string"
+            || name.starts_with("string.")
+            || name == "character"
+            || name.starts_with("character.")
+        {
+            string_capture_ids.push(idx as u32);
+        }
+
         // Rust uses 'label' for lifetimes, and highglighting
         // lifetimes consistently with types seems reasonable.
-        type_capture_ids.push(idx);
-    }
-    if let Some(idx) = config.highlight_query.capture_index_for_name("tag") {
-        type_capture_ids.push(idx);
+        if name == "label" {
+            type_capture_ids.push(idx as u32);
+        }
     }
 
     let mut qc = ts::QueryCursor::new();
@@ -550,11 +784,18 @@ fn print_cursor(src: &str, cursor: &mut ts::TreeCursor, depth: usize) {
         let node = cursor.node();
         node.end_position();
 
+        let formatted_node = format!(
+            "{} {} - {}",
+            node.kind().replace('\n', "\\n"),
+            node.start_position(),
+            node.end_position()
+        );
+
         if node.child_count() == 0 {
             let node_src = &src[node.start_byte()..node.end_byte()];
-            println!("{}{:#?} {:?}", "  ".repeat(depth), node, node_src);
+            println!("{}{} {:?}", "  ".repeat(depth), formatted_node, node_src);
         } else {
-            println!("{}{:#?}", "  ".repeat(depth), node);
+            println!("{}{}", "  ".repeat(depth), formatted_node,);
         }
 
         if cursor.goto_first_child() {
@@ -663,7 +904,7 @@ fn all_syntaxes_from_cursor<'a>(
     let mut result: Vec<&Syntax> = vec![];
 
     loop {
-        result.push(syntax_from_cursor(
+        result.extend(syntax_from_cursor(
             arena, src, nl_pos, cursor, config, highlights,
         ));
 
@@ -684,14 +925,14 @@ fn syntax_from_cursor<'a>(
     cursor: &mut ts::TreeCursor,
     config: &TreeSitterConfig,
     highlights: &HighlightedNodeIds,
-) -> &'a Syntax<'a> {
+) -> Vec<&'a Syntax<'a>> {
     let node = cursor.node();
 
     if node.is_error() {
         let position = nl_pos.from_offsets(node.start_byte(), node.end_byte());
         let content = &src[node.start_byte()..node.end_byte()];
 
-        warn!(
+        debug!(
             "Tree-sitter syntax error at {:?}: {}",
             position.get(0),
             content
@@ -703,7 +944,9 @@ fn syntax_from_cursor<'a>(
         // of whether they have children.
         atom_from_cursor(arena, src, nl_pos, cursor, highlights)
     } else if node.child_count() > 0 {
-        list_from_cursor(arena, src, nl_pos, cursor, config, highlights)
+        vec![list_from_cursor(
+            arena, src, nl_pos, cursor, config, highlights,
+        )]
     } else {
         atom_from_cursor(arena, src, nl_pos, cursor, highlights)
     }
@@ -760,21 +1003,21 @@ fn list_from_cursor<'a>(
     loop {
         let node = cursor.node();
         if node_i < i {
-            before_delim.push(syntax_from_cursor(
+            before_delim.extend(syntax_from_cursor(
                 arena, src, nl_pos, cursor, config, highlights,
             ));
         } else if node_i == i {
             inner_open_content = &src[node.start_byte()..node.end_byte()];
             inner_open_position = nl_pos.from_offsets(node.start_byte(), node.end_byte());
         } else if node_i < j {
-            between_delim.push(syntax_from_cursor(
+            between_delim.extend(syntax_from_cursor(
                 arena, src, nl_pos, cursor, config, highlights,
             ));
         } else if node_i == j {
             inner_close_content = &src[node.start_byte()..node.end_byte()];
             inner_close_position = nl_pos.from_offsets(node.start_byte(), node.end_byte());
         } else if node_i > j {
-            after_delim.push(syntax_from_cursor(
+            after_delim.extend(syntax_from_cursor(
                 arena, src, nl_pos, cursor, config, highlights,
             ));
         }
@@ -827,11 +1070,19 @@ fn atom_from_cursor<'a>(
     nl_pos: &NewlinePositions,
     cursor: &mut ts::TreeCursor,
     highlights: &HighlightedNodeIds,
-) -> &'a Syntax<'a> {
+) -> Vec<&'a Syntax<'a>> {
     let node = cursor.node();
     let position = nl_pos.from_offsets(node.start_byte(), node.end_byte());
     let mut content = &src[node.start_byte()..node.end_byte()];
 
+    // The C and C++ grammars have a '\n' node with the
+    // preprocessor. This isn't useful for difftastic, because it's
+    // not visible, but leads us to highlight unchanged lines that
+    // happen to have preceding newline node.
+    if node.kind() == "\n" {
+        return vec![];
+    }
+
     // JSX trims whitespace at the beginning and end of text nodes.
     // TODO: match the exact trimming behaviour used in React.
     //
@@ -853,7 +1104,7 @@ fn atom_from_cursor<'a>(
         AtomKind::Normal
     };
 
-    Syntax::new_atom(arena, position, content, highlight)
+    vec![Syntax::new_atom(arena, position, content, highlight)]
 }
 
 #[cfg(test)]
